@@ -30,6 +30,7 @@ import numpy
 # local imports
 from nion.data import Core
 from nion.data import DataAndMetadata
+from nion.swift.model import Activity
 from nion.swift.model import DataItem
 from nion.swift.model import DisplayItem
 from nion.swift.model import Graphics
@@ -45,13 +46,27 @@ from nion.utils.ReferenceCounting import weak_partial
 if typing.TYPE_CHECKING:
     from nion.swift.model import DocumentModel
 
-DataElementType = typing.Dict[str, typing.Any]
-_FinishedCallbackType = typing.Callable[[typing.Sequence[typing.Optional[DataAndMetadata.DataAndMetadata]]], None]
-_NDArray = numpy.typing.NDArray[typing.Any]
-
 _ = gettext.gettext
 
 
+class DataAndMetadataPromise:
+    def __init__(self, xdata: typing.Optional[DataAndMetadata.DataAndMetadata]) -> None:
+        self.__xdata_unsafe = xdata
+        self.__xdata: typing.Optional[DataAndMetadata.DataAndMetadata] = None
+
+    @property
+    def xdata(self) -> typing.Optional[DataAndMetadata.DataAndMetadata]:
+        if not self.__xdata and self.__xdata_unsafe:
+            self.__xdata = copy.deepcopy(self.__xdata_unsafe)
+            self.__xdata_unsafe = None
+        return self.__xdata
+
+
+DataElementType = typing.Dict[str, typing.Any]
+_FinishedCallbackType = typing.Callable[[typing.Sequence[DataAndMetadataPromise]], None]
+_NDArray = numpy.typing.NDArray[typing.Any]
+
+
 class DocumentModelInterface(typing.Protocol):
 
     @abc.abstractmethod
@@ -91,18 +106,14 @@ class HardwareSourceBridge:
         self.__data_channel_start_listeners: typing.Dict[str, typing.List[Event.EventListener]] = dict()
         self.__data_channel_stop_listeners: typing.Dict[str, typing.List[Event.EventListener]] = dict()
         self.__data_channel_states_updated_listeners: typing.Dict[str, Event.EventListener] = dict()
-        self.__hardware_source_call_soon_event_listeners: typing.Dict[str, Event.EventListener] = dict()
         self.__hardware_source_added_listener = HardwareSourceManager().hardware_source_added_event.listen(self.__hardware_source_added)
         self.__hardware_source_removed_listener = HardwareSourceManager().hardware_source_removed_event.listen(self.__hardware_source_removed)
         for hardware_source in HardwareSourceManager().hardware_sources:
             self.__hardware_source_added(hardware_source)
-        self.__project_loaded_event_listener = document_model.project_loaded_event.listen(weak_partial(HardwareSourceBridge.clean_display_items, self))
 
     def close(self) -> None:
         # close hardware source related stuff
         # close data items left to append that haven't been appended
-        self.__project_loaded_event_listener.close()
-        self.__project_loaded_event_listener = typing.cast(typing.Any, None)
         with self.__data_items_to_append_lock:
             for key, data_item in self.__data_items_to_append:
                 data_item.close()
@@ -126,26 +137,13 @@ class HardwareSourceBridge:
         self.__data_channel_updated_listeners = typing.cast(typing.Any, None)
         self.__data_channel_start_listeners = typing.cast(typing.Any, None)
         self.__data_channel_stop_listeners = typing.cast(typing.Any, None)
-        for listener in self.__hardware_source_call_soon_event_listeners.values():
-            listener.close()
-        self.__hardware_source_call_soon_event_listeners = typing.cast(typing.Any, None)
         self.__document_model = typing.cast(typing.Any, None)
 
-    def clean_display_items(self) -> None:
-        # clean the display items for each data channel
-        for hardware_source in HardwareSourceManager().hardware_sources:
-            for data_channel in hardware_source.data_channels:
-                data_item_reference = self.__document_model.get_data_item_channel_reference(hardware_source.hardware_source_id, data_channel.channel_id)
-                data_item = data_item_reference.data_item
-                if data_item:
-                    hardware_source.clean_display_items(self, list(self.__document_model.get_display_items_for_data_item(data_item)))
-
     def __call_soon(self, fn: typing.Callable[[], None]) -> None:
         assert self.__document_model
         self.__document_model._call_soon(fn)
 
     def __hardware_source_added(self, hardware_source: HardwareSource) -> None:
-        self.__hardware_source_call_soon_event_listeners[hardware_source.hardware_source_id] = hardware_source.call_soon_event.listen(self.__call_soon)
         self.__data_channel_states_updated_listeners[hardware_source.hardware_source_id] = hardware_source.data_channel_states_updated.listen(functools.partial(self.__data_channel_states_updated, hardware_source))
         for data_channel in hardware_source.data_channels:
             data_channel_updated_listener = data_channel.data_channel_updated_event.listen(functools.partial(self.__data_channel_updated, hardware_source, data_channel))
@@ -154,11 +152,8 @@ class HardwareSourceBridge:
             self.__data_channel_start_listeners.setdefault(hardware_source.hardware_source_id, list()).append(data_channel_start_listener)
             data_channel_stop_listener = data_channel.data_channel_stop_event.listen(functools.partial(self.__data_channel_stop, hardware_source, data_channel))
             self.__data_channel_stop_listeners.setdefault(hardware_source.hardware_source_id, list()).append(data_channel_stop_listener)
-            # NOTE: clean_display_items is called in __finish_project_read
 
     def __hardware_source_removed(self, hardware_source: HardwareSource) -> None:
-        self.__hardware_source_call_soon_event_listeners[hardware_source.hardware_source_id].close()
-        del self.__hardware_source_call_soon_event_listeners[hardware_source.hardware_source_id]
         self.__data_channel_states_updated_listeners[hardware_source.hardware_source_id].close()
         del self.__data_channel_states_updated_listeners[hardware_source.hardware_source_id]
         for listener in self.__data_channel_updated_listeners.get(hardware_source.hardware_source_id, list()):
@@ -240,25 +235,14 @@ class HardwareSourceBridge:
         self.__document_model._queue_data_item_update(data_item, data_and_metadata)
 
     def __data_channel_states_updated(self, hardware_source: HardwareSource, data_channels: typing.Sequence[DataChannel]) -> None:
-        data_item_states = list()
+        channel_map: typing.Dict[str, DataItem.DataItem] = dict()
         for data_channel in data_channels:
             data_item_reference = self.__document_model.get_data_item_channel_reference(hardware_source.hardware_source_id, data_channel.channel_id)
             data_item = data_item_reference.data_item
             channel_id = data_channel.channel_id
-            channel_data_state = data_channel.state
-            sub_area = data_channel.sub_area
-            # make sure to send out the complete frame
-            data_item_state: typing.Dict[str, typing.Any] = dict()
-            if channel_id is not None:
-                data_item_state["channel_id"] = channel_id
-            data_item_state["data_item"] = data_item
-            data_item_state["channel_state"] = channel_data_state
-            if sub_area:
-                data_item_state["sub_area"] = sub_area
-            data_item_states.append(data_item_state)
-        # temporary until things get cleaned up
-        hardware_source.data_item_states_changed_event.fire(data_item_states)
-        hardware_source.data_item_states_changed(data_item_states)
+            if channel_id and data_item:
+                channel_map[channel_id] = data_item
+        hardware_source.data_channel_map_updated(channel_map)
 
 
 # Keeps track of all registered hardware sources and instruments.
@@ -449,16 +433,22 @@ class AcquisitionTask:
         self.__minimum_period = 1 / 1000.0
         self.__frame_index = 0
         self.__view_id = str(uuid.uuid4()) if not continuous else None
-        self._test_acquire_exception = None
-        self._test_acquire_hook = None
+        self._test_acquire_exception: typing.Optional[typing.Callable[[Exception], None]] = None
+        self._test_start_hook: typing.Optional[typing.Callable[[], None]] = None
+        self._test_acquire_hook: typing.Optional[typing.Callable[[], None]] = None
         self.start_event = Event.Event()
         self.stop_event = Event.Event()
         self.data_elements_changed_event = Event.Event()
         self.finished_callback_fn: typing.Optional[_FinishedCallbackType] = None
+        self.activity: typing.Optional[Activity.Activity] = None
 
     def __mark_as_finished(self) -> None:
         self.__finished = True
-        self.data_elements_changed_event.fire(list(), self.__view_id, False, self.__is_stopping)
+        self.data_elements_changed_event.fire(list(), self.__view_id, False, self.__is_stopping, False)
+
+    def __mark_as_error(self) -> None:
+        self.__finished = True
+        self.data_elements_changed_event.fire(list(), self.__view_id, False, self.__is_stopping, True)
 
     # called from the hardware source
     # note: abort, suspend and execute are always called from the same thread, ensuring that
@@ -470,7 +460,7 @@ class AcquisitionTask:
                 self.__start()
             except Exception as e:
                 # the task is finished if it doesn't start
-                self.__mark_as_finished()
+                self.__mark_as_error()
                 raise
             self.__started = True
             # logging.debug("%s started", self)
@@ -500,8 +490,8 @@ class AcquisitionTask:
             except Exception as e:
                 # the task is finished if it doesn't execute
                 # logging.debug("exception")
-                self._stop_acquisition()
-                self.__mark_as_finished()
+                self._safe_stop_acquisition()
+                self.__mark_as_error()
                 raise
 
     # called from the hardware source
@@ -529,6 +519,9 @@ class AcquisitionTask:
         self._mark_acquisition()
 
     def __start(self) -> None:
+        if callable(self._test_start_hook):
+            self._test_start_hook()
+
         if not self._start_acquisition():
             self.abort()
         self.__last_acquire_time = time.time() - self.__minimum_period
@@ -539,7 +532,7 @@ class AcquisitionTask:
         elapsed = time.time() - self.__last_acquire_time
         time.sleep(max(0.0, self.__minimum_period - elapsed))
 
-        if self._test_acquire_hook:
+        if callable(self._test_acquire_hook):
             self._test_acquire_hook()
 
         partial_data_elements = self._acquire_data_elements()
@@ -565,7 +558,7 @@ class AcquisitionTask:
                 break
 
         # notify that data elements have changed. at this point data_elements may contain data stored in low level code.
-        self.data_elements_changed_event.fire(data_elements, self.__view_id, complete, self.__is_stopping)
+        self.data_elements_changed_event.fire(data_elements, self.__view_id, complete, self.__is_stopping, False)
 
         if complete:
             self.__frame_index += 1
@@ -629,6 +622,17 @@ class AcquisitionTask:
     def _stop_acquisition(self) -> None:
         self.stop_event.fire()
 
+    def _safe_stop_acquisition(self) -> None:
+        try:
+            self._stop_acquisition()
+        except Exception as e:
+            if callable(self._test_acquire_exception):
+                self._test_acquire_exception(e)
+            else:
+                import traceback
+                logging.debug(f"STOP Error: {e}")
+                traceback.print_exc()
+
     # subclasses are expected to implement this function efficiently since it will
     # be repeatedly called. in practice that means that subclasses MUST sleep (directly
     # or indirectly) unless the data is immediately available, which it shouldn't be on
@@ -683,9 +687,11 @@ class DataChannel:
         self.__dest_sub_area: typing.Optional[Geometry.IntRect] = None
         self.__data_and_metadata: typing.Optional[DataAndMetadata.DataAndMetadata] = None
         self.is_dirty = False
+        self.__is_error = False
         self.data_channel_updated_event = Event.Event()
         self.data_channel_start_event = Event.Event()
         self.data_channel_stop_event = Event.Event()
+        self.data_channel_state_changed_event = Event.Event()
 
     @property
     def index(self) -> int:
@@ -727,12 +733,23 @@ class DataChannel:
     def data_and_metadata(self) -> typing.Optional[DataAndMetadata.DataAndMetadata]:
         return self.__data_and_metadata
 
+    @property
+    def is_error(self) -> bool:
+        return self.__is_error
+
+    @is_error.setter
+    def is_error(self, value: bool) -> None:
+        if value != self.__is_error:
+            self.__is_error = value
+            self.data_channel_state_changed_event.fire()
+
     @property
     def is_started(self) -> bool:
         return self.__start_count > 0
 
     def update(self, data_and_metadata: DataAndMetadata.DataAndMetadata, state: str, data_shape: typing.Optional[DataAndMetadata.ShapeType], dest_sub_area: typing.Optional[Geometry.IntRectTuple], sub_area: typing.Optional[Geometry.IntRectTuple], view_id: typing.Optional[str]) -> None:
         """Called from hardware source when new data arrives."""
+        old_state = self.__state
         self.__state = state
         self.__data_shape = data_shape or data_and_metadata.data_shape
 
@@ -794,18 +811,24 @@ class DataChannel:
         self.data_channel_updated_event.fire(new_extended_data)
         self.is_dirty = True
 
+        if old_state != self.__state:
+            self.data_channel_state_changed_event.fire()
+
     def start(self) -> None:
         """Called from hardware source when data starts streaming."""
         old_start_count = self.__start_count
         self.__start_count += 1
         if old_start_count == 0:
+            self.is_error = False
             self.data_channel_start_event.fire()
+            self.data_channel_state_changed_event.fire()
 
     def stop(self) -> None:
         """Called from hardware source when data stops streaming."""
         self.__start_count -= 1
         if self.__start_count == 0:
             self.data_channel_stop_event.fire()
+            self.data_channel_state_changed_event.fire()
 
 
 class Instrument(typing.Protocol):
@@ -825,7 +848,7 @@ class HardwareSource(typing.Protocol):
     def start_playing(self, *args: typing.Any, **kwargs: typing.Any) -> None: ...
     def abort_playing(self, *, sync_timeout: typing.Optional[float] = None) -> None: ...
     def stop_playing(self, *, sync_timeout: typing.Optional[float] = None) -> None: ...
-    def start_recording(self, sync_timeout: typing.Optional[float] = None, finished_callback_fn: typing.Optional[_FinishedCallbackType] = None) -> None: ...
+    def start_recording(self, sync_timeout: typing.Optional[float] = None, finished_callback_fn: typing.Optional[_FinishedCallbackType] = None, *, frame_parameters: typing.Optional[FrameParameters] = None, **kwargs: typing.Any) -> None: ...
     def abort_recording(self, sync_timeout: typing.Optional[float] = None) -> None: ...
     def stop_recording(self, sync_timeout: typing.Optional[float] = None) -> None: ...
     def get_next_xdatas_to_finish(self, timeout: typing.Optional[float] = None) -> typing.Sequence[typing.Optional[DataAndMetadata.DataAndMetadata]]: ...
@@ -873,18 +896,16 @@ class HardwareSource(typing.Protocol):
     # private. do not use outside of instrumentation-kit.
 
     data_channel_states_updated: Event.Event
+    data_channel_state_changed_event: Event.Event
     xdatas_available_event: Event.Event
     abort_event: Event.Event
     acquisition_state_changed_event: Event.Event
-    data_item_states_changed_event: Event.Event
-    call_soon_event: Event.Event
 
     def add_data_channel(self, channel_id: typing.Optional[str] = None, name: typing.Optional[str] = None) -> None: ...
     def add_channel_processor(self, channel_index: int, processor: SumProcessor) -> None: ...
-    def clean_display_items(self, document_model: HardwareSourceBridge, display_items: typing.Sequence[DisplayItem.DisplayItem], **kwargs: typing.Any) -> None: ...
     def get_frame_parameters_from_dict(self, d: typing.Mapping[str, typing.Any]) -> FrameParameters: ...
     def set_channel_enabled(self, channel_index: int, enabled: bool) -> None: ...
-    def data_item_states_changed(self, data_item_states: typing.Sequence[typing.Mapping[str, typing.Any]]) -> None: ...
+    def data_channel_map_updated(self, data_channel_map: typing.Mapping[str, DataItem.DataItem]) -> None: ...
     def set_record_frame_parameters(self, frame_parameters: FrameParameters) -> None: ...
     def get_record_frame_parameters(self) -> FrameParameters: ...
 
@@ -900,13 +921,13 @@ class ConcreteHardwareSource(Observable.Observable, HardwareSource):
         self.__hardware_source_id = hardware_source_id
         self.__display_name = display_name
         self.__data_channels: typing.List[DataChannel] = list()
+        self.__data_channel_state_changed_listeners: typing.List[Event.EventListener] = list()
         self.__features: typing.Dict[str, typing.Any] = dict()
         self.data_channel_states_updated = Event.Event()
         self.xdatas_available_event = Event.Event()
         self.abort_event = Event.Event()
         self.acquisition_state_changed_event = Event.Event()
-        self.data_item_states_changed_event = Event.Event()
-        self.call_soon_event = Event.Event()
+        self.data_channel_state_changed_event = Event.Event()
         self.__break_for_closing = False
         self.__acquire_thread_trigger = threading.Event()
         self.__tasks: typing.Dict[str, AcquisitionTask] = dict()
@@ -916,11 +937,13 @@ class ConcreteHardwareSource(Observable.Observable, HardwareSource):
         self.__acquire_thread: typing.Optional[threading.Thread] = threading.Thread(target=self.__acquire_thread_loop)
         self.__acquire_thread.daemon = True
         self.__acquire_thread.start()
-        self._test_acquire_exception = None
-        self._test_acquire_hook = None
+        self._test_acquire_exception: typing.Optional[typing.Callable[[Exception], None]] = None
+        self._test_start_hook: typing.Optional[typing.Callable[[], None]] = None
+        self._test_acquire_hook: typing.Optional[typing.Callable[[], None]] = None
 
     def close(self) -> None:
         self.close_thread()
+        self.__data_channel_state_changed_listeners = typing.cast(typing.Any, None)
 
     @property
     def features(self) -> typing.Dict[str, typing.Any]:
@@ -954,9 +977,6 @@ class ConcreteHardwareSource(Observable.Observable, HardwareSource):
             self.__acquire_thread.join()
             self.__acquire_thread = None
 
-    def _call_soon(self, fn: typing.Callable[[], None]) -> None:
-        self.call_soon_event.fire_any(fn)
-
     def __acquire_thread_loop(self) -> None:
         # acquire_thread_trigger should be set whenever the task list change.
         while self.__acquire_thread_trigger.wait():
@@ -997,6 +1017,10 @@ class ConcreteHardwareSource(Observable.Observable, HardwareSource):
                         logging.debug("{} Error: {}".format(task_id.capitalize(), e))
                         traceback.print_exc()
                 if task.is_finished:
+                    activity = self.__tasks[task_id].activity
+                    if activity:
+                        Activity.activity_finished(activity)
+                    self.__tasks[task_id].activity = None
                     del self.__tasks[task_id]
                     self.__data_elements_changed_event_listeners[task_id].close()
                     del self.__data_elements_changed_event_listeners[task_id]
@@ -1009,9 +1033,8 @@ class ConcreteHardwareSource(Observable.Observable, HardwareSource):
             if break_for_closing:
                 break
 
-    # subclasses can implement this method which is called when the data items used for acquisition change.
-    # NOTE: this is called from DocumentModel!
-    def data_item_states_changed(self, data_item_states: typing.Sequence[typing.Mapping[str, typing.Any]]) -> None:
+    # subclasses can implement this method which is called when the data channels are updated.
+    def data_channel_map_updated(self, data_channel_map: typing.Mapping[str, DataItem.DataItem]) -> None:
         pass
 
     # subclasses should implement this method to create a continuous-style acquisition task.
@@ -1030,7 +1053,7 @@ class ConcreteHardwareSource(Observable.Observable, HardwareSource):
     # subclasses should implement this method to create a non-continuous-style acquisition task.
     # create the view task
     # will be called from the UI thread and should return quickly.
-    def _create_acquisition_record_task(self) -> AcquisitionTask:
+    def _create_acquisition_record_task(self, *, frame_parameters: typing.Optional[FrameParameters] = None, **kwargs: typing.Any) -> AcquisitionTask:
         raise NotImplementedError()
 
     # subclasses can implement this method to get notification that the record task has been changed.
@@ -1040,7 +1063,7 @@ class ConcreteHardwareSource(Observable.Observable, HardwareSource):
     def _record_task_updated(self, record_task: typing.Optional[AcquisitionTask]) -> None:
         pass
 
-    def __data_elements_changed(self, task: AcquisitionTask, data_elements: typing.Sequence[DataElementType], view_id: typing.Optional[str], is_complete: bool, is_stopping: bool) -> None:
+    def __data_elements_changed(self, task: AcquisitionTask, data_elements: typing.Sequence[DataElementType], view_id: typing.Optional[str], is_complete: bool, is_stopping: bool, is_error: bool) -> None:
         """Called in response to a data_elements_changed event from the task.
 
         data_elements is a list of data_elements; may be an empty list
@@ -1076,8 +1099,8 @@ class ConcreteHardwareSource(Observable.Observable, HardwareSource):
             channel_index = next((data_channel.index for data_channel in self.__data_channels if data_channel.channel_id == channel_id), 0)
             data_and_metadata = ImportExportManager.convert_data_element_to_data_and_metadata(data_element)
             # data_and_metadata data may still point to low level code memory at this point.
-            channel_state = data_element.get("state", "complete")
-            if channel_state != "complete" and is_stopping:
+            channel_state = data_element.get("state", "complete" if not is_error else "error")
+            if channel_state != "complete" and is_stopping and not is_error:
                 channel_state = "marked"
             data_shape = data_element.get("data_shape")
             dest_sub_area = data_element.get("dest_sub_area")
@@ -1092,24 +1115,38 @@ class ConcreteHardwareSource(Observable.Observable, HardwareSource):
             src_channel_index = data_channel.src_channel_index
             if src_channel_index is not None:
                 src_data_channel = self.__data_channels[src_channel_index]
-                src_data_and_metadata = src_data_channel.data_and_metadata
-                data_channel_processor = data_channel.processor
-                if data_channel_processor and src_data_and_metadata and src_data_channel.is_dirty and src_data_channel.state == "complete":
-                    processed_data_and_metadata = data_channel_processor.process(src_data_and_metadata)
-                    data_channel.update(processed_data_and_metadata, "complete", None, None, None, view_id)
-                data_channels.append(data_channel)
-                xdatas.append(data_channel.data_and_metadata)
+                if src_data_channel in data_channels:
+                    src_data_and_metadata = src_data_channel.data_and_metadata
+                    data_channel_processor = data_channel.processor
+                    if not is_error:
+                        if data_channel_processor and src_data_and_metadata and src_data_channel.is_dirty and src_data_channel.state == "complete":
+                            processed_data_and_metadata = data_channel_processor.process(src_data_and_metadata)
+                            data_channel.update(processed_data_and_metadata, "complete", None, None, None, view_id)
+                    else:
+                        assert data_channel.data_and_metadata
+                        data_channel.update(data_channel.data_and_metadata, "error", None, None, None, view_id)
+                    data_channels.append(data_channel)
+                    xdatas.append(data_channel.data_and_metadata)
         # all channel buffers are clean now
         for data_channel in self.__data_channels:
+            data_channel.is_error = is_error
             data_channel.is_dirty = False
 
         self.data_channel_states_updated.fire(data_channels)
+
         if is_complete:
             # xdatas are may still be pointing to memory in low level code here
-            self.xdatas_available_event.fire(xdatas)
+            # send promises which will give access to the data, but not copy it unless it is used.
+            data_promises = [DataAndMetadataPromise(xdata) for xdata in xdatas]
+            self.xdatas_available_event.fire(data_promises)
             # hack to allow record to know when its data is finished
             if callable(task.finished_callback_fn):
-                task.finished_callback_fn(xdatas)
+                task.finished_callback_fn(data_promises)
+
+        if is_error:
+            from nion.swift.model import Notification
+            Notification.notify(Notification.Notification("nion.acquisition.error", "\N{WARNING SIGN} Acquisition",
+                                                          "Acquisition Failed", "Acquisition has failed due to an error."))
 
     def __start(self) -> None:
         for data_channel in self.__data_channels:
@@ -1118,6 +1155,7 @@ class ConcreteHardwareSource(Observable.Observable, HardwareSource):
     def __stop(self) -> None:
         for data_channel in self.__data_channels:
             data_channel.stop()
+        self.data_channel_states_updated.fire(list())
 
     # return whether task is running
     def is_task_running(self, task_id: str) -> bool:
@@ -1132,6 +1170,8 @@ class ConcreteHardwareSource(Observable.Observable, HardwareSource):
         self.__data_elements_changed_event_listeners[task_id] = task.data_elements_changed_event.listen(functools.partial(self.__data_elements_changed, task))
         self.__start_event_listeners[task_id] = task.start_event.listen(self.__start)
         self.__stop_event_listeners[task_id] = task.stop_event.listen(self.__stop)
+        task.activity = Activity.Activity(self.hardware_source_id + "_" + task_id, f"{self.display_name} ({task_id})")
+        Activity.append_activity(task.activity)
         self.__tasks[task_id] = task
         # TODO: sync the thread start by waiting for an event on the task which gets set when the acquire thread starts executing the task
         self.__acquire_thread_trigger.set()
@@ -1162,6 +1202,7 @@ class ConcreteHardwareSource(Observable.Observable, HardwareSource):
     def start_playing(self, *args: typing.Any, **kwargs: typing.Any) -> None:
         if not self.is_playing:
             view_task = self._create_acquisition_view_task()
+            view_task._test_start_hook = self._test_start_hook
             view_task._test_acquire_hook = self._test_acquire_hook
             self._view_task_updated(view_task)
             self.start_task('view', view_task)
@@ -1202,16 +1243,16 @@ class ConcreteHardwareSource(Observable.Observable, HardwareSource):
 
     # call this to start acquisition
     # thread safe
-    def start_recording(self, sync_timeout: typing.Optional[float] = None, finished_callback_fn: typing.Optional[_FinishedCallbackType] = None) -> None:
+    def start_recording(self, sync_timeout: typing.Optional[float] = None, finished_callback_fn: typing.Optional[_FinishedCallbackType] = None, *, frame_parameters: typing.Optional[FrameParameters] = None, **kwargs: typing.Any) -> None:
         if not self.is_recording:
-            record_task = self._create_acquisition_record_task()
+            record_task = self._create_acquisition_record_task(frame_parameters=frame_parameters)
             old_finished_callback_fn = record_task.finished_callback_fn
 
-            def finished(xdatas: typing.Sequence[typing.Optional[DataAndMetadata.DataAndMetadata]]) -> None:
+            def finished(data_promises: typing.Sequence[DataAndMetadataPromise]) -> None:
                 if callable(old_finished_callback_fn):
-                    old_finished_callback_fn(xdatas)
+                    old_finished_callback_fn(data_promises)
                 if callable(finished_callback_fn):
-                    finished_callback_fn(xdatas)
+                    finished_callback_fn(data_promises)
 
             record_task.finished_callback_fn = finished
             self._record_task_updated(record_task)
@@ -1250,8 +1291,8 @@ class ConcreteHardwareSource(Observable.Observable, HardwareSource):
         new_data_event = threading.Event()
         new_xdatas: typing.List[typing.Optional[DataAndMetadata.DataAndMetadata]] = list()
 
-        def receive_new_xdatas(xdatas: typing.Sequence[typing.Optional[DataAndMetadata.DataAndMetadata]]) -> None:
-            new_xdatas[:] = xdatas
+        def receive_new_xdatas(data_promises: typing.Sequence[DataAndMetadataPromise]) -> None:
+            new_xdatas[:] = [data_promise.xdata for data_promise in data_promises]
             new_data_event.set()
 
         def abort() -> None:
@@ -1269,8 +1310,8 @@ class ConcreteHardwareSource(Observable.Observable, HardwareSource):
         new_data_event = threading.Event()
         new_xdatas: typing.List[typing.Optional[DataAndMetadata.DataAndMetadata]] = list()
 
-        def receive_new_xdatas(xdatas: typing.Sequence[typing.Optional[DataAndMetadata.DataAndMetadata]]) -> None:
-            new_xdatas[:] = xdatas
+        def receive_new_xdatas(data_promises: typing.Sequence[DataAndMetadataPromise]) -> None:
+            new_xdatas[:] = [data_promise.xdata for data_promise in data_promises]
             new_data_event.set()
 
         def abort() -> None:
@@ -1297,19 +1338,18 @@ class ConcreteHardwareSource(Observable.Observable, HardwareSource):
     def data_channels(self) -> typing.Sequence[DataChannel]:
         return self.__data_channels
 
+    def __data_channel_state_changed(self, data_channel: DataChannel) -> None:
+        self.data_channel_state_changed_event.fire(data_channel)
+
     def add_data_channel(self, channel_id: typing.Optional[str] = None, name: typing.Optional[str] = None) -> None:
-        self.__data_channels.append(DataChannel(self, len(self.__data_channels), channel_id, name))
+        data_channel = DataChannel(self, len(self.__data_channels), channel_id, name)
+        self.__data_channels.append(data_channel)
+        self.__data_channel_state_changed_listeners.append(data_channel.data_channel_state_changed_event.listen(weak_partial(ConcreteHardwareSource.__data_channel_state_changed, self, data_channel)))
 
     def add_channel_processor(self, channel_index: int, processor: SumProcessor) -> None:
-        self.__data_channels.append(DataChannel(self, len(self.__data_channels), processor.processor_id, None, channel_index, processor))
-
-    def clean_display_items(self, document_model: HardwareSourceBridge, display_items: typing.Sequence[DisplayItem.DisplayItem], **kwargs: typing.Any) -> None:
-        """Clean the display items associated with this data channel.
-
-        Invoked when the hardware source is registered with the document model. Useful for
-        removing old graphics and otherwise cleaning up the data item at startup.
-        """
-        pass
+        data_channel = DataChannel(self, len(self.__data_channels), processor.processor_id, None, channel_index, processor)
+        self.__data_channels.append(data_channel)
+        self.__data_channel_state_changed_listeners.append(data_channel.data_channel_state_changed_event.listen(weak_partial(ConcreteHardwareSource.__data_channel_state_changed, self, data_channel)))
 
     def get_property(self, name: str) -> typing.Any:
         return getattr(self, name)
@@ -1562,9 +1602,6 @@ class RecordTask:
 
         assert not self.__hardware_source.is_recording
 
-        if frame_parameters:
-            self.__hardware_source.set_record_frame_parameters(frame_parameters)
-
         self.__data_and_metadata_list: typing.Sequence[typing.Optional[DataAndMetadata.DataAndMetadata]] = list()
         # synchronize start of thread; if this sync doesn't occur, the task can be closed before the acquisition
         # is started. in that case a deadlock occurs because the abort doesn't apply and the thread is waiting
@@ -1572,6 +1609,7 @@ class RecordTask:
         self.__recording_started = threading.Event()
 
         def record_thread() -> None:
+            self.__hardware_source.set_record_frame_parameters(frame_parameters)
             self.__hardware_source.start_recording()
             self.__recording_started.set()
             self.__data_and_metadata_list = self.__hardware_source.get_next_xdatas_to_finish()
@@ -1797,9 +1835,6 @@ class MetadataDisplayComponent:
             d["info_items"] = info_items
 
 
-Registry.register_component(MetadataDisplayComponent(), {"metadata_display"})
-
-
 def matches_hardware_source(hardware_source_id: str, channel_id: typing.Optional[str], document_model: DocumentModel.DocumentModel, data_item: DataItem.DataItem) -> bool:
     if not document_model.get_data_item_computation(data_item):
         hardware_source_metadata = data_item.metadata.get("hardware_source", dict())
@@ -1809,8 +1844,6 @@ def matches_hardware_source(hardware_source_id: str, channel_id: typing.Optional
     return False
 
 
-Registry.register_component(HardwareSourceManager(), {"hardware_source_manager"})
-
 hardware_source_bridges: typing.Dict[uuid.UUID, HardwareSourceBridge] = dict()
 
 
@@ -1827,5 +1860,22 @@ def handle_component_unregistered(component: typing.Any, component_types: typing
         hardware_source_bridges.pop(component.uuid).close()
 
 
-component_registered_event_listener = Registry.listen_component_registered_event(handle_component_registered)
-component_unregistered_event_listener = Registry.listen_component_unregistered_event(handle_component_unregistered)
+_component_registered_event_listener = None
+_component_unregistered_event_listener = None
+
+
+def run() -> None:
+    Registry.register_component(MetadataDisplayComponent(), {"metadata_display"})
+    Registry.register_component(HardwareSourceManager(), {"hardware_source_manager"})
+    global _component_registered_event_listener
+    global _component_unregistered_event_listener
+    _component_registered_event_listener = Registry.listen_component_registered_event(handle_component_registered)
+    _component_unregistered_event_listener = Registry.listen_component_unregistered_event(handle_component_unregistered)
+
+def stop() -> None:
+    Registry.unregister_component(Registry.get_component("metadata_display"), {"metadata_display"})
+    Registry.unregister_component(Registry.get_component("hardware_source_manager"), {"hardware_source_manager"})
+    global _component_registered_event_listener
+    global _component_unregistered_event_listener
+    _component_registered_event_listener = None
+    _component_unregistered_event_listener = None
