@@ -3,21 +3,28 @@ import copy
 import json
 import random
 import time
+import typing
 import unittest
 import zlib
 
 import numpy
 
+from nion.data import DataAndMetadata
+from nion.instrumentation import AcquisitionPreferences
 from nion.instrumentation import camera_base
 from nion.instrumentation.test import AcquisitionTestContext
 from nion.instrumentation.test import HardwareSource_test
 from nion.swift import Application
+from nion.swift import DocumentController
 from nion.swift import Facade
 from nion.swift.model import Graphics
 from nion.swift.model import Metadata
+from nion.swift.model import Schema
 from nion.ui import TestUI
 from nion.utils import Geometry
+from nion.utils import Model
 from nion.utils import Registry
+from nionswift_plugin.nion_instrumentation_ui import AcquisitionPanel
 from nionswift_plugin.nion_instrumentation_ui import CameraControlPanel
 
 """
@@ -31,6 +38,68 @@ suite.run(result)
 
 TIMEOUT = 20
 
+
+class ApplicationDataInMemory:
+    def __init__(self) -> None:
+        self.d = dict()
+
+    def get_data_dict(self) -> typing.Dict[str, typing.Any]:
+        return self.d
+
+    def set_data_dict(self, d: typing.Mapping[str, typing.Any]) -> None:
+        self.d = dict(d)
+
+
+def make_camera_device(test_context: AcquisitionTestContext.test_context) -> AcquisitionPanel.AcquisitionDeviceResult:
+    return AcquisitionPanel.build_camera_device_data_stream(test_context.camera_hardware_source)
+
+
+def make_scan_device(test_context: AcquisitionTestContext.test_context) -> AcquisitionPanel.AcquisitionDeviceResult:
+    return AcquisitionPanel.build_scan_device_data_stream(test_context.scan_hardware_source)
+
+
+def make_synchronized_device(test_context: AcquisitionTestContext.test_context) -> AcquisitionPanel.AcquisitionDeviceResult:
+    scan_context_description = AcquisitionPanel.SynchronizedScanDescription(str(), True, str(), Geometry.IntSize(6, 4), 0)
+    return AcquisitionPanel.build_synchronized_device_data_stream(test_context.scan_hardware_source, scan_context_description, test_context.camera_hardware_source)
+
+
+def make_sequence_acquisition_method(adr: AcquisitionPanel.AcquisitionDeviceResult) -> AcquisitionPanel.AcquisitionMethodResult:
+    return AcquisitionPanel.wrap_acquisition_device_data_stream_for_sequence(adr.data_stream, 4, adr.channel_names)
+
+
+def make_series_acquisition_method(adr: AcquisitionPanel.AcquisitionDeviceResult) -> AcquisitionPanel.AcquisitionMethodResult:
+    control_customization = AcquisitionPreferences.ControlCustomization(Schema.get_entity_type("control_customization"), None)
+    control_customization._set_field_value("control_id", "defocus")
+    control_customization.device_control_id = "C10"
+    control_customization.delay = 0
+    control_values_range = AcquisitionPanel.ControlValuesRange(4, 500e-9, 5e-9)
+    return AcquisitionPanel.wrap_acquisition_device_data_stream_for_series(
+        adr.data_stream,
+        control_customization,
+        control_values_range,
+        adr.device_map,
+        adr.channel_names
+    )
+
+
+def make_tableau_acquisition_method(adr: AcquisitionPanel.AcquisitionDeviceResult) -> AcquisitionPanel.AcquisitionMethodResult:
+    control_customization = AcquisitionPreferences.ControlCustomization(Schema.get_entity_type("control_customization"), None)
+    control_customization._set_field_value("control_id", "stage_position")
+    control_customization.device_control_id = "stage_position_m"
+    control_customization.delay = 0
+    x_control_values_range = AcquisitionPanel.ControlValuesRange(3, -1e-9, 1e-9)
+    y_control_values_range = AcquisitionPanel.ControlValuesRange(3, -1e-9, 1e-9)
+    return AcquisitionPanel.wrap_acquisition_device_data_stream_for_tableau(
+        adr.data_stream,
+        control_customization,
+        "tv",
+        x_control_values_range,
+        y_control_values_range,
+        adr.device_map,
+        adr.channel_names
+    )
+
+
 class TestCameraControlClass(unittest.TestCase):
 
     def setUp(self):
@@ -38,7 +107,7 @@ class TestCameraControlClass(unittest.TestCase):
         self.source_image = numpy.random.randn(1024, 1024).astype(numpy.float32)
         self.exposure = 1.0
 
-    def _acquire_one(self, document_controller, hardware_source):
+    def _acquire_one(self, document_controller: DocumentController.DocumentController, hardware_source: camera_base.CameraHardwareSource) -> None:
         hardware_source.start_playing()
         try:
             start_time = time.time()
@@ -132,6 +201,20 @@ class TestCameraControlClass(unittest.TestCase):
             self._acquire_one(document_controller, hardware_source)
             self.assertEqual(len(document_model.data_items), 1)
 
+    def test_record_acquires_properly_binned_data(self):
+        with self.__test_context() as test_context:
+            for binning in (1, 2):
+                hardware_source = test_context.camera_hardware_source
+                frame_parameters = hardware_source.get_frame_parameters(2)
+                frame_parameters.binning = binning
+                hardware_source.set_record_frame_parameters(frame_parameters)
+                hardware_source.start_recording(sync_timeout=3.0)
+                try:
+                    results = hardware_source.get_next_xdatas_to_finish()
+                    self.assertEqual(results[0].data.shape, hardware_source.get_expected_dimensions(binning))
+                finally:
+                    hardware_source.stop_recording(sync_timeout=3.0)
+
     def test_ability_to_set_profile_parameters_is_reflected_in_acquisition(self):
         with self.__test_context() as test_context:
             document_controller = test_context.document_controller
@@ -649,24 +732,28 @@ class TestCameraControlClass(unittest.TestCase):
 
     def test_consecutive_frames_have_unique_data(self):
         # this test will fail if the camera is saturated (or otherwise produces identical values naturally)
+        numpy_random_state = numpy.random.get_state()
+        random_state = random.getstate()
         numpy.random.seed(999)
         random.seed(999)
-        self.source_image = numpy.random.randn(1024, 1024).astype(numpy.float32)
-        with self.__test_context() as test_context:
-            hardware_source = test_context.camera_hardware_source
-            hardware_source.start_playing()
-            try:
-                data = hardware_source.get_next_xdatas_to_finish()[0].data
-                last_hash = zlib.crc32(data)
-                for _ in range(16):
+        try:
+            self.source_image = numpy.random.randn(1024, 1024).astype(numpy.float32)
+            with self.__test_context() as test_context:
+                hardware_source = test_context.camera_hardware_source
+                hardware_source.start_playing()
+                try:
                     data = hardware_source.get_next_xdatas_to_finish()[0].data
-                    next_hash = zlib.crc32(data)
-                    self.assertNotEqual(last_hash, next_hash)
-                    last_hash = next_hash
-            finally:
-                hardware_source.abort_playing()
-            numpy.random.seed()
-            random.seed()
+                    last_hash = zlib.crc32(data)
+                    for _ in range(16):
+                        data = hardware_source.get_next_xdatas_to_finish()[0].data
+                        next_hash = zlib.crc32(data)
+                        self.assertNotEqual(last_hash, next_hash)
+                        last_hash = next_hash
+                finally:
+                    hardware_source.abort_playing()
+        finally:
+            random.setstate(random_state)
+            numpy.random.set_state(numpy_random_state)
 
     def test_integrating_frames_updates_frame_count_by_integration_count(self):
         with self.__test_context() as test_context:
@@ -944,6 +1031,150 @@ class TestCameraControlClass(unittest.TestCase):
             # test dict is writeable to json
             json.dumps(frame_parameters.as_dict())
 
+    def test_acquisition_state_updates_during_acquisition(self):
+        with self.__test_context() as test_context:
+            document_controller = test_context.document_controller
+            hardware_source = test_context.camera_hardware_source
+            state_controller = self.__create_state_controller(test_context)
+            self.assertEqual(state_controller.acquisition_state_model.value, "stopped")
+            hardware_source.start_playing()
+            try:
+                hardware_source.get_next_xdatas_to_finish(5)
+                document_controller.periodic()
+                self.assertIn(state_controller.acquisition_state_model.value, ("partial", "complete"))
+            finally:
+                hardware_source.stop_playing(sync_timeout=TIMEOUT)
+                document_controller.periodic()
+            self.assertEqual(state_controller.acquisition_state_model.value, "stopped")
+
+    def test_acquisition_state_after_exception_during_start(self):
+        with self.__test_context() as test_context:
+            document_controller = test_context.document_controller
+            hardware_source = test_context.camera_hardware_source
+            def raise_exception():
+                raise Exception("Error during acquisition")
+            hardware_source._test_start_hook = raise_exception
+            hardware_source._test_acquire_exception = lambda *args: None
+            state_controller = self.__create_state_controller(test_context)
+            hardware_source.start_playing()
+            try:
+                hardware_source.get_next_xdatas_to_finish(5)
+                document_controller.periodic()
+            finally:
+                hardware_source.stop_playing(sync_timeout=TIMEOUT)
+            document_controller.periodic()
+            self.assertEqual(state_controller.acquisition_state_model.value, "error")
+
+    def test_acquisition_state_after_exception_during_execute(self):
+        with self.__test_context() as test_context:
+            document_controller = test_context.document_controller
+            hardware_source = test_context.camera_hardware_source
+            def raise_exception():
+                raise Exception("Error during acquisition")
+            hardware_source._test_acquire_hook = raise_exception
+            hardware_source._test_acquire_exception = lambda *args: None
+            state_controller = self.__create_state_controller(test_context)
+            hardware_source.start_playing()
+            try:
+                hardware_source.get_next_xdatas_to_finish(5)
+                document_controller.periodic()
+            finally:
+                hardware_source.stop_playing(sync_timeout=TIMEOUT)
+            document_controller.periodic()
+            self.assertEqual(state_controller.acquisition_state_model.value, "error")
+
+    def test_acquisition_panel_sequence_acquisition(self):
+        # this test is complicated to set up because it is testing the UI.
+        with self.__test_context() as test_context:
+            document_controller = test_context.document_controller
+            hardware_source = test_context.camera_hardware_source
+            c = Schema.Entity(Schema.get_entity_type("acquisition_device_component_camera"))
+            c.camera_device_id = hardware_source.hardware_source_id
+            c2 = Schema.Entity(Schema.get_entity_type("acquisition_method_component_sequence_acquire"))
+            c2.count = 4
+            acquisition_configuration = AcquisitionPanel.AcquisitionConfiguration(ApplicationDataInMemory())
+            acquisition_configuration.acquisition_device_component_id = "camera"
+            acquisition_configuration.acquisition_method_component_id = "sequence-acquire"
+            acquisition_preferences = AcquisitionPreferences.AcquisitionPreferences(ApplicationDataInMemory())
+            ac = AcquisitionPanel.AcquisitionController(document_controller, acquisition_configuration, acquisition_preferences)
+            with contextlib.closing(ac):
+                h = AcquisitionPanel.CameraAcquisitionDeviceComponentHandler(c, acquisition_preferences)
+                with contextlib.closing(h):
+                    h2 = AcquisitionPanel.SequenceAcquisitionMethodComponentHandler(c2, acquisition_preferences)
+                    with contextlib.closing(h2):
+                        adr = h.build_acquisition_device_data_stream()
+                        amr = h2.wrap_acquisition_device_data_stream(adr.data_stream, adr.device_map, adr.channel_names)
+                        ac._acquire_data_stream(amr.data_stream, amr.title_base, amr.channel_names, adr.drift_tracker)
+                        while ac.is_acquiring_model.value:
+                            document_controller.periodic()
+                        self.assertFalse(ac.is_error)
+            # only one data item will be create: the sequence. the view data item does not exist since acquiring
+            # a sequence will use the special sequence acquisition of the camera device.
+            self.assertEqual(1, len(document_controller.document_model.data_items))
+
+    def __test_acq(self, document_controller: DocumentController.DocumentController, adr: AcquisitionPanel.AcquisitionDeviceResult, amr: AcquisitionPanel.AcquisitionMethodResult, expected_dimensions: typing.Sequence[typing.Tuple[DataAndMetadata.ShapeType, DataAndMetadata.DataDescriptor]]) -> None:
+        acquisition_state = AcquisitionPanel.AcquisitionState()
+        progress_value_model = Model.PropertyModel[int](0)
+        is_acquiring_model = Model.PropertyModel[bool](False)
+        AcquisitionPanel._acquire_data_stream(amr.data_stream,
+                                              document_controller,
+                                              acquisition_state,
+                                              progress_value_model,
+                                              is_acquiring_model,
+                                              amr.title_base,
+                                              amr.channel_names,
+                                              adr.drift_tracker)
+        while is_acquiring_model.value:
+            document_controller.periodic()
+        self.assertFalse(acquisition_state.is_error)
+        self.assertEqual(len(expected_dimensions), len(document_controller.document_model.data_items))
+        for data_item, expected_dimension in zip(document_controller.document_model.data_items, expected_dimensions):
+            self.assertEqual(expected_dimension[0], data_item.data_shape)
+            self.assertEqual(expected_dimension[1], data_item.data_and_metadata.data_descriptor)
+
+    def test_acquisition_panel_acquisition(self):
+        tc = [
+            # only one data item will be created: the sequence. the view data item does not exist since acquiring
+            # a sequence will use the special sequence acquisition of the camera device.
+            (make_camera_device, make_sequence_acquisition_method, [((4, 1024, 1024), DataAndMetadata.DataDescriptor(True, 0, 2))]),
+
+            # two data items will be created: the series and the camera view.
+            (make_camera_device, make_series_acquisition_method, [((4, 1024, 1024), DataAndMetadata.DataDescriptor(True, 0, 2)),
+                                                                  ((1024, 1024), DataAndMetadata.DataDescriptor(False, 0, 2))]),
+
+            # two data items will be created: the series and the camera view.
+            (make_camera_device, make_tableau_acquisition_method, [((3, 3, 1024, 1024), DataAndMetadata.DataDescriptor(False, 2, 2)),
+                                                                   ((1024, 1024), DataAndMetadata.DataDescriptor(False, 0, 2))]),
+
+            # two data items will be created: the series and the scan view.
+            (make_scan_device, make_sequence_acquisition_method, [((4, 256, 256), DataAndMetadata.DataDescriptor(True, 0, 2)),
+                                                                  ((256, 256), DataAndMetadata.DataDescriptor(False, 0, 2))]),
+
+            # two data items will be created: the series and the scan view.
+            (make_scan_device, make_series_acquisition_method, [((4, 256, 256), DataAndMetadata.DataDescriptor(True, 0, 2)),
+                                                                ((256, 256), DataAndMetadata.DataDescriptor(False, 0, 2))]),
+
+            # two data items will be created: the tableau and the scan view.
+            (make_scan_device, make_tableau_acquisition_method, [((3, 3, 256, 256), DataAndMetadata.DataDescriptor(False, 2, 2)),
+                                                                 ((256, 256), DataAndMetadata.DataDescriptor(False, 0, 2))]),
+
+            # three data items will be created: the camera series, the sync'd series, and the scan view.
+            (make_synchronized_device, make_sequence_acquisition_method, [((4, 6, 4), DataAndMetadata.DataDescriptor(True, 0, 2)),
+                                                                          ((4, 6, 4, 1024, 1024), DataAndMetadata.DataDescriptor(True, 2, 2)),
+                                                                          ((6, 4), DataAndMetadata.DataDescriptor(False, 0, 2))]),
+
+            # not supported yet; no way to represent it as a single data item.
+            # (make_synchronized_device, make_series_acquisition_method, [((4, 6, 4), DataAndMetadata.DataDescriptor(True, 0, 2)),
+            #                                                             ((4, 6, 4, 1024, 1024), DataAndMetadata.DataDescriptor(True, 2, 2)),
+            #                                                             ((6, 4), DataAndMetadata.DataDescriptor(False, 0, 2))]),
+        ]
+        for adr_fn, amr_fn, expected_count in tc:
+            with self.subTest(adr_fn=adr_fn, amr_fn=amr_fn, expected_count=expected_count):
+                with self.__test_context() as test_context:
+                    document_controller = test_context.document_controller
+                    adr = adr_fn(test_context)
+                    amr = amr_fn(adr)
+                    self.__test_acq(document_controller, adr, amr, expected_count)
 
     def planned_test_custom_view_followed_by_ui_view_uses_ui_frame_parameters(self):
         pass
