@@ -66,7 +66,7 @@ class SimpleHardwareSource(HardwareSource.ConcreteHardwareSource):
     def _create_acquisition_view_task(self) -> SimpleAcquisitionTask:
         return SimpleAcquisitionTask(True, self.sleep, self.image)
 
-    def _create_acquisition_record_task(self) -> SimpleAcquisitionTask:
+    def _create_acquisition_record_task(self, **kwargs) -> SimpleAcquisitionTask:
         return SimpleAcquisitionTask(False, self.sleep, self.image)
 
 
@@ -114,7 +114,7 @@ class LinePlotHardwareSource(HardwareSource.ConcreteHardwareSource):
     def _create_acquisition_view_task(self) -> LinePlotAcquisitionTask:
         return LinePlotAcquisitionTask(self.shape, True, self.sleep)
 
-    def _create_acquisition_record_task(self) -> LinePlotAcquisitionTask:
+    def _create_acquisition_record_task(self, **kwargs) -> LinePlotAcquisitionTask:
         return LinePlotAcquisitionTask(self.shape, False, self.sleep)
 
 
@@ -131,7 +131,7 @@ class SummedHardwareSource(HardwareSource.ConcreteHardwareSource):
     def _create_acquisition_view_task(self) -> SimpleAcquisitionTask:
         return SimpleAcquisitionTask(True, self.sleep, self.image)
 
-    def _create_acquisition_record_task(self) -> SimpleAcquisitionTask:
+    def _create_acquisition_record_task(self, **kwargs) -> SimpleAcquisitionTask:
         return SimpleAcquisitionTask(False, self.sleep, self.image)
 
 
@@ -259,7 +259,7 @@ class ScanHardwareSource(HardwareSource.ConcreteHardwareSource):
     def _create_acquisition_view_task(self) -> ScanAcquisitionTask:
         return ScanAcquisitionTask(True, self.sleep, self.channel_enabled_list, self.scanning_ref, self.suspended_ref, self.suspend_event, self.image)
 
-    def _create_acquisition_record_task(self) -> ScanAcquisitionTask:
+    def _create_acquisition_record_task(self, **kwargs) -> ScanAcquisitionTask:
         return ScanAcquisitionTask(False, self.sleep, self.channel_enabled_list, self.scanning_ref, self.suspended_ref, self.suspend_event, self.image)
 
 
@@ -431,9 +431,8 @@ def _test_exception_during_record_halts_playback(testcase, hardware_source, expo
     hardware_source.start_recording(sync_timeout=3.0)
     try:
         start = time.time()
-        while time.time() - start < exposure * 10.0 and hardware_source.is_recording:
-            time.sleep(0.05)
-        # print(time.time() - start)
+        while time.time() - start < max(exposure * 10.0, 3.0) and hardware_source.is_recording:
+            time.sleep(0.01)  # 10 msec
         testcase.assertFalse(hardware_source.is_recording)
     finally:
         hardware_source.abort_recording(sync_timeout=3.0)
@@ -442,9 +441,8 @@ def _test_exception_during_record_halts_playback(testcase, hardware_source, expo
     hardware_source.start_recording()
     try:
         start = time.time()
-        while time.time() - start < exposure * 10.0 and hardware_source.is_recording:
-            time.sleep(0.05)
-        time.sleep(0.05)  # avoid test race condition
+        while time.time() - start < max(exposure * 10.0, 3.0) and hardware_source.is_recording:
+            time.sleep(0.01)  # 10 msec
         testcase.assertFalse(hardware_source.is_recording)
     finally:
         hardware_source.abort_recording(sync_timeout=3.0)
@@ -517,6 +515,7 @@ class TestHardwareSourceClass(unittest.TestCase):
         class SimpleTestContext(TestContext.MemoryProfileContext):
             def __init__(self) -> None:
                 super().__init__()
+                HardwareSource.run()
                 self.document_controller = self.create_document_controller(auto_close=False)
                 self.document_model = self.document_controller.document_model
                 self.hardware_source = SimpleHardwareSource()
@@ -524,7 +523,9 @@ class TestHardwareSourceClass(unittest.TestCase):
                 HardwareSource.HardwareSourceManager().register_hardware_source(self.hardware_source)
 
             def close(self) -> None:
+                self.document_controller.periodic()
                 self.document_controller.close()
+                HardwareSource.stop()
                 super().close()
 
         return SimpleTestContext()
@@ -547,12 +548,19 @@ class TestHardwareSourceClass(unittest.TestCase):
         class LinePlotTestContext(TestContext.MemoryProfileContext):
             def __init__(self) -> None:
                 super().__init__()
-                self.document_controller = self.create_document_controller()
+                HardwareSource.run()
+                self.document_controller = self.create_document_controller(auto_close=False)
                 self.document_model = self.document_controller.document_model
                 self.hardware_source = LinePlotHardwareSource(shape, processed)
                 self.hardware_source.exposure = 0.01
                 HardwareSource.HardwareSourceManager().register_hardware_source(self.hardware_source)
 
+            def close(self) -> None:
+                self.document_controller.periodic()
+                self.document_controller.close()
+                HardwareSource.stop()
+                super().close()
+
         return LinePlotTestContext()
 
     def __scan_test_context(self):
@@ -560,7 +568,8 @@ class TestHardwareSourceClass(unittest.TestCase):
         class ScanTestContext(TestContext.MemoryProfileContext):
             def __init__(self) -> None:
                 super().__init__()
-                self.document_controller = self.create_document_controller()
+                HardwareSource.run()
+                self.document_controller = self.create_document_controller(auto_close=False)
                 self.document_model = self.document_controller.document_model
                 self.hardware_source = ScanHardwareSource()
                 self.hardware_source.exposure = 0.01
@@ -569,6 +578,12 @@ class TestHardwareSourceClass(unittest.TestCase):
                 self.hardware_source.positioned = False
                 HardwareSource.HardwareSourceManager().register_hardware_source(self.hardware_source)
 
+            def close(self) -> None:
+                self.document_controller.periodic()
+                self.document_controller.close()
+                HardwareSource.stop()
+                super().close()
+
         return ScanTestContext()
 
     def test_registering_and_unregistering_works_as_expected(self):
@@ -687,13 +702,11 @@ class TestHardwareSourceClass(unittest.TestCase):
     def test_exception_during_record_halts_scan(self):
         with self.__scan_test_context() as scan_test_context:
             hardware_source = scan_test_context.hardware_source
-            document_controller = scan_test_context.document_controller
             _test_exception_during_record_halts_playback(self, hardware_source, hardware_source.sleep)
 
     def test_able_to_restart_scan_after_exception_scan(self):
         with self.__scan_test_context() as scan_test_context:
             hardware_source = scan_test_context.hardware_source
-            document_controller = scan_test_context.document_controller
             _test_able_to_restart_view_after_exception(self, hardware_source, hardware_source.sleep)
 
     def test_record_starts_and_finishes_in_reasonable_time(self):
@@ -797,7 +810,7 @@ class TestHardwareSourceClass(unittest.TestCase):
             document_model.append_data_item(data_item)
             document_model.setup_channel(document_model.make_data_item_reference_key(hardware_source.hardware_source_id, "a"), data_item)
             hardware_source.data_channels[0].update(DataAndMetadata.new_data_and_metadata(data), "complete", None, None, None, None)
-            hardware_source.sleep = 0.10
+            hardware_source.sleep = 0.20
             hardware_source.start_playing()
             time.sleep(0.02)
             hardware_source.abort_playing(sync_timeout=3.0)
