@@ -28,6 +28,7 @@ from nion.ui import Declarative
 from nion.ui import Dialog
 from nion.ui import Widgets
 from nion.utils import Geometry
+from nion.utils import Model
 from nion.utils import Registry
 
 if typing.TYPE_CHECKING:
@@ -40,37 +41,19 @@ if typing.TYPE_CHECKING:
     from nion.ui import Window
     from nion.utils import Event
 
-
 _ = gettext.gettext
 
+map_channel_state_to_text = {
+    "stopped": _("Stopped"),
+    "complete": _("Acquiring"),
+    "partial": _("Acquiring"),
+    "marked": _("Stopping"),
+    "error": _("Error"),
+}
+
 
 class CameraControlStateController:
-    """
-    Track the state of a camera controller, as it relates to the UI. This object does not hold any state itself.
-
-    Camera controller should support the following API:
-        (acquisition)
-            (event) data_item_states_changed_event(data_item_states)
-            (event) acquisition_state_changed_event(is_acquiring)
-            (read-only property) hardware_source_id
-            (read-only property) is_playing
-            (read-only property) display_name
-            (read-only property) features
-            (method) start_playing()
-            (method) stop_playing()
-            (method) abort_playing()
-        (event) profile_changed_event(profile_index)
-        (event) frame_parameters_changed_event(profile_index, frame_parameters)
-        (event) log_messages_event(messages, data_elements)
-        (read-only property) selected_profile_index: return current profile index
-        (method) set_selected_profile_index(profile_index): change the profile index
-        (method) get_frame_parameters(profile_index)
-        (method) set_frame_parameters(profile_index, frame_parameters)
-        (method) set_current_frame_parameters(frame_parameters)
-        (method) get_current_frame_parameters()
-        (method) shift_click(mouse_position, camera_shape)
-        (method) open_configuration_interface(api_broker)
-        (method) periodic()
+    """Track the state of a camera controller, as it relates to the UI. This object does not hold any state itself.
 
     Clients can query:
         (property) has_processed_data
@@ -100,7 +83,6 @@ class CameraControlStateController:
         on_capture_button_state_changed(visible, enabled)
         on_display_new_data_item(data_item)
         on_log_messages(messages, data_elements)
-        (thread) on_data_item_states_changed(data_item_states)
     """
 
     def __init__(self, camera_hardware_source: camera_base.CameraHardwareSource, queue_task: typing.Callable[[typing.Callable[[], None]], None], document_model: DocumentModel.DocumentModel) -> None:
@@ -111,9 +93,9 @@ class CameraControlStateController:
         self.__document_model = document_model
         self.__profile_changed_event_listener: typing.Optional[Event.EventListener] = None
         self.__frame_parameters_changed_event_listener: typing.Optional[Event.EventListener] = None
-        self.__data_item_states_changed_event_listener: typing.Optional[Event.EventListener] = None
         self.__acquisition_state_changed_event_listener: typing.Optional[Event.EventListener] = None
         self.__log_messages_event_listener: typing.Optional[Event.EventListener] = None
+        self.__data_channel_state_changed_event_listener: typing.Optional[Event.EventListener] = None
         self.on_display_name_changed: typing.Optional[typing.Callable[[str], None]] = None
         self.on_binning_values_changed: typing.Optional[typing.Callable[[typing.Sequence[int]], None]] = None
         self.on_profiles_changed: typing.Optional[typing.Callable[[typing.Sequence[str]], None]] = None
@@ -121,7 +103,7 @@ class CameraControlStateController:
         self.on_frame_parameters_changed: typing.Optional[typing.Callable[[camera_base.CameraFrameParameters], None]] = None
         self.on_play_button_state_changed: typing.Optional[typing.Callable[[bool, str], None]] = None
         self.on_abort_button_state_changed: typing.Optional[typing.Callable[[bool, bool], None]] = None
-        self.on_data_item_states_changed: typing.Optional[typing.Callable[[typing.Sequence[typing.Mapping[str, typing.Any]]], None]] = None
+        self.acquisition_state_model = Model.PropertyModel[str]("stopped")
         self.on_capture_button_state_changed: typing.Optional[typing.Callable[[bool, bool], None]] = None
         self.on_display_new_data_item: typing.Optional[typing.Callable[[DataItem.DataItem], None]] = None
         self.on_camera_current_changed: typing.Optional[typing.Callable[[typing.Optional[float]], None]] = None
@@ -152,9 +134,7 @@ class CameraControlStateController:
         if self.__acquisition_state_changed_event_listener:
             self.__acquisition_state_changed_event_listener.close()
             self.__acquisition_state_changed_event_listener = None
-        if self.__data_item_states_changed_event_listener:
-            self.__data_item_states_changed_event_listener.close()
-            self.__data_item_states_changed_event_listener = None
+        self.__data_channel_state_changed_event_listener = None
         if self.__log_messages_event_listener:
             self.__log_messages_event_listener.close()
             self.__log_messages_event_listener = None
@@ -165,7 +145,6 @@ class CameraControlStateController:
         self.on_frame_parameters_changed = None
         self.on_play_button_state_changed = None
         self.on_abort_button_state_changed = None
-        self.on_data_item_states_changed = None
         self.on_capture_button_state_changed = None
         self.on_display_new_data_item = None
         self.on_log_messages = None
@@ -215,31 +194,32 @@ class CameraControlStateController:
     def has_processed_data(self) -> bool:
         return self.__has_processed_channel
 
-    def __receive_new_xdatas(self, xdatas: typing.Sequence[DataAndMetadata.DataAndMetadata]) -> None:
+    def __receive_new_xdatas(self, data_promises: typing.Sequence[HardwareSource.DataAndMetadataPromise]) -> None:
         current_time = time.time()
-        if current_time - self.__last_camera_current_time > 5.0 and len(xdatas) > 0 and callable(self.on_camera_current_changed):
-            xdata = xdatas[0]
-            counts_per_electron = xdata.metadata.get("hardware_source", dict()).get("counts_per_electron")
-            exposure = xdata.metadata.get("hardware_source", dict()).get("exposure")
-            if xdata.intensity_calibration.units == "counts" and counts_per_electron and exposure:
-                sum_counts = xdata.intensity_calibration.convert_to_calibrated_value(numpy.sum(xdatas[0]._data_ex))
-                detector_current = sum_counts / exposure / counts_per_electron / 6.242e18 if exposure > 0 and counts_per_electron > 0 else 0.0
-                if detector_current != self.__camera_current:
-                    self.__camera_current = detector_current
+        if current_time - self.__last_camera_current_time > 5.0 and len(data_promises) > 0 and callable(self.on_camera_current_changed):
+            xdata = data_promises[0].xdata
+            if xdata:
+                counts_per_electron = xdata.metadata.get("hardware_source", dict()).get("counts_per_electron")
+                exposure = xdata.metadata.get("hardware_source", dict()).get("exposure")
+                if xdata.intensity_calibration.units == "counts" and counts_per_electron and exposure:
+                    sum_counts = xdata.intensity_calibration.convert_to_calibrated_value(numpy.sum(xdata._data_ex))
+                    detector_current = sum_counts / exposure / counts_per_electron / 6.242e18 if exposure > 0 and counts_per_electron > 0 else 0.0
+                    if detector_current != self.__camera_current:
+                        self.__camera_current = detector_current
 
-                    def update_camera_current() -> None:
-                        if callable(self.on_camera_current_changed):
-                            self.on_camera_current_changed(self.__camera_current)
+                        def update_camera_current() -> None:
+                            if callable(self.on_camera_current_changed):
+                                self.on_camera_current_changed(self.__camera_current)
 
-                    self.queue_task(update_camera_current)
-            self.__last_camera_current_time = current_time
+                        self.queue_task(update_camera_current)
+                self.__last_camera_current_time = current_time
 
     def initialize_state(self) -> None:
         """ Call this to initialize the state of the UI after everything has been connected. """
         if self.__camera_hardware_source:
             self.__profile_changed_event_listener = self.__camera_hardware_source.profile_changed_event.listen(self.__update_profile_index)
             self.__frame_parameters_changed_event_listener = self.__camera_hardware_source.frame_parameters_changed_event.listen(self.__update_frame_parameters)
-            self.__data_item_states_changed_event_listener = self.__camera_hardware_source.data_item_states_changed_event.listen(self.__data_item_states_changed)
+            self.__data_channel_state_changed_event_listener = self.__camera_hardware_source.data_channel_state_changed_event.listen(self.__data_channel_state_changed)
             self.__acquisition_state_changed_event_listener = self.__camera_hardware_source.acquisition_state_changed_event.listen(self.__acquisition_state_changed)
             self.__log_messages_event_listener = self.__camera_hardware_source.log_messages_event.listen(self.__log_messages)
         if callable(self.on_display_name_changed):
@@ -251,8 +231,6 @@ class CameraControlStateController:
             profile_items = self.__camera_hardware_source.modes
             self.on_profiles_changed(profile_items)
             self.__update_profile_index(self.__camera_hardware_source.selected_profile_index)
-        if callable(self.on_data_item_states_changed):
-            self.on_data_item_states_changed(list())
 
     # must be called on ui thread
     def handle_change_profile(self, profile_label: str) -> None:
@@ -295,6 +273,7 @@ class CameraControlStateController:
     def handle_binning_changed(self, binning_str: str) -> None:
         frame_parameters = self.__camera_hardware_source.get_frame_parameters(self.__camera_hardware_source.selected_profile_index)
         frame_parameters.binning = max(int(binning_str), 1)
+        frame_parameters = self.__camera_hardware_source.validate_frame_parameters(frame_parameters)
         self.__camera_hardware_source.set_frame_parameters(self.__camera_hardware_source.selected_profile_index, frame_parameters)
 
     # must be called on ui thread
@@ -304,35 +283,40 @@ class CameraControlStateController:
             frame_parameters.exposure_ms = float(exposure)
         except ValueError:
             pass
+        frame_parameters = self.__camera_hardware_source.validate_frame_parameters(frame_parameters)
         self.__camera_hardware_source.set_frame_parameters(self.__camera_hardware_source.selected_profile_index, frame_parameters)
 
     def handle_decrease_exposure(self) -> None:
         frame_parameters = self.__camera_hardware_source.get_frame_parameters(self.__camera_hardware_source.selected_profile_index)
         frame_parameters.exposure_ms = frame_parameters.exposure_ms * 0.5
+        frame_parameters = self.__camera_hardware_source.validate_frame_parameters(frame_parameters)
         self.__camera_hardware_source.set_frame_parameters(self.__camera_hardware_source.selected_profile_index, frame_parameters)
 
     def handle_increase_exposure(self) -> None:
         frame_parameters = self.__camera_hardware_source.get_frame_parameters(self.__camera_hardware_source.selected_profile_index)
         frame_parameters.exposure_ms = frame_parameters.exposure_ms * 2.0
+        frame_parameters = self.__camera_hardware_source.validate_frame_parameters(frame_parameters)
         self.__camera_hardware_source.set_frame_parameters(self.__camera_hardware_source.selected_profile_index, frame_parameters)
 
     def handle_capture_clicked(self) -> None:
-        def capture_xdatas(xdatas: typing.Sequence[DataAndMetadata.DataAndMetadata]) -> None:
+        def capture_xdatas(data_promises: typing.Sequence[HardwareSource.DataAndMetadataPromise]) -> None:
             if self.__captured_xdatas_available_event:
                 self.__captured_xdatas_available_event.close()
                 self.__captured_xdatas_available_event = None
-            for index, xdata in enumerate(xdatas):
+            for index, data_promise in enumerate(data_promises):
                 def add_data_item(data_item: DataItem.DataItem) -> None:
                     self.__document_model.append_data_item(data_item)
                     if callable(self.on_display_new_data_item):
                         self.on_display_new_data_item(data_item)
 
                 if index == (1 if self.use_processed_data else 0):
-                    data_item = DataItem.new_data_item(xdata)
-                    display_name = xdata.metadata.get("hardware_source", dict()).get("hardware_source_name")
-                    display_name = display_name if display_name else _("Capture")
-                    data_item.title = display_name
-                    self.queue_task(functools.partial(add_data_item, data_item))
+                    xdata = data_promise.xdata
+                    if xdata:
+                        data_item = DataItem.new_data_item(xdata)
+                        display_name = xdata.metadata.get("hardware_source", dict()).get("hardware_source_name")
+                        display_name = display_name if display_name else _("Capture")
+                        data_item.title = display_name
+                        self.queue_task(functools.partial(add_data_item, data_item))
             self.queue_task(self.__update_buttons)
 
         self.__captured_xdatas_available_event = self.__camera_hardware_source.xdatas_available_event.listen(capture_xdatas)
@@ -364,10 +348,11 @@ class CameraControlStateController:
         if callable(self.on_log_messages):
             self.on_log_messages(messages, data_elements)
 
-    # this message comes from the hardware source. may be called from thread.
-    def __data_item_states_changed(self, data_item_states: typing.Sequence[typing.Mapping[str, typing.Any]]) -> None:
-        if callable(self.on_data_item_states_changed):
-            self.on_data_item_states_changed(data_item_states)
+    def __data_channel_state_changed(self, data_channel: HardwareSource.DataChannel) -> None:
+        if data_channel.is_started and data_channel.state:
+            self.acquisition_state_model.value = data_channel.state
+        else:
+            self.acquisition_state_model.value = "error" if data_channel.is_error else "stopped"
 
 
 class IconCanvasItem(CanvasItem.TextButtonCanvasItem):
@@ -840,15 +825,14 @@ class CameraControlWidget(Widgets.CompositeWidgetBase):
             # abort_button.visible = visible
             abort_button.enabled = enabled
 
-        def data_item_states_changed(data_item_states: typing.Sequence[typing.Mapping[str, typing.Any]]) -> None:
-            map_channel_state_to_text = {"stopped": _("Stopped"), "complete": _("Acquiring"),
-                                         "partial": _("Acquiring"), "marked": _("Stopping")}
-            if len(data_item_states) > 0:
-                data_item_state = data_item_states[0]
-                channel_state = data_item_state["channel_state"]
-                play_state_label.text = map_channel_state_to_text[channel_state]
-            else:
-                play_state_label.text = map_channel_state_to_text["stopped"]
+        def acquisition_state_changed(key: str) -> None:
+            # this may be called on a thread. create an async method (guaranteed to run on the main thread)
+            # and add it to the window event loop.
+            async def update_acquisition_state_label(acquisition_state: typing.Optional[str]) -> None:
+                acquisition_state = acquisition_state or "stopped"
+                play_state_label.text = map_channel_state_to_text[acquisition_state]
+
+            self.document_controller.event_loop.create_task(update_acquisition_state_label(self.__state_controller.acquisition_state_model.value))
 
         def camera_current_changed(camera_current: typing.Optional[float]) -> None:
             if camera_current:
@@ -873,12 +857,15 @@ class CameraControlWidget(Widgets.CompositeWidgetBase):
         self.__state_controller.on_frame_parameters_changed = frame_parameters_changed
         self.__state_controller.on_play_button_state_changed = play_button_state_changed
         self.__state_controller.on_abort_button_state_changed = abort_button_state_changed
-        self.__state_controller.on_data_item_states_changed = lambda a: self.document_controller.queue_task(lambda: data_item_states_changed(a))
         self.__state_controller.on_camera_current_changed = camera_current_changed
         self.__state_controller.on_log_messages = log_messages
 
+        self.__acquisition_state_changed_listener = self.__state_controller.acquisition_state_model.property_changed_event.listen(acquisition_state_changed)
+
         self.__state_controller.initialize_state()
 
+        acquisition_state_changed("value")
+
     # HACK: this is used to dump log messages to Swift.
     def periodic(self) -> None:
         self.__state_controller.handle_periodic()
@@ -896,6 +883,7 @@ class CameraControlWidget(Widgets.CompositeWidgetBase):
         self.__image_display_mouse_released_event_listener = typing.cast(typing.Any, None)
         self.__state_controller.close()
         self.__state_controller = typing.cast(typing.Any, None)
+        self.__acquisition_state_changed_listener = typing.cast(typing.Any, None)
         super().close()
 
     # this gets called from the DisplayPanelManager. pass on the message to the state controller.
@@ -1013,7 +1001,6 @@ class CameraDisplayPanelController:
         self.__play_button_play_button_state = "play"
         self.__abort_button_visible = False
         self.__abort_button_enabled = False
-        self.__data_item_states: typing.List[typing.Mapping[str, typing.Any]] = list()
         self.__display_panel = display_panel
         self.__display_panel.header_canvas_item.end_header_color = "#98FB98"
         self.__playback_controls_composition = CanvasItem.CanvasItemComposition()
@@ -1070,16 +1057,15 @@ class CameraDisplayPanelController:
                 abort_button_canvas_item.enabled = abort_button_enabled
                 abort_button_canvas_item.size_to_content(display_panel.image_panel_get_font_metrics)
 
-        def update_status_text() -> None:
-            map_channel_state_to_text = {"stopped": _("Stopped"), "complete": _("Acquiring"),
-                                         "partial": _("Acquiring"), "marked": _("Stopping")}
-            for data_item_state in self.__data_item_states:
-                channel_state = data_item_state["channel_state"]
-                new_text = map_channel_state_to_text[channel_state]
-                if status_text_canvas_item.text != new_text:
-                    status_text_canvas_item.text = new_text
-                    status_text_canvas_item.size_to_content(display_panel.image_panel_get_font_metrics)
-                return
+        def acquisition_state_changed(key: str) -> None:
+            # this may be called on a thread. create an async method (guaranteed to run on the main thread)
+            # and add it to the window event loop.
+            async def update_acquisition_state_label(acquisition_state: typing.Optional[str]) -> None:
+                acquisition_state = acquisition_state or "stopped"
+                status_text_canvas_item.text = map_channel_state_to_text[acquisition_state]
+                status_text_canvas_item.size_to_content(display_panel.image_panel_get_font_metrics)
+
+            self.__display_panel.document_controller.event_loop.create_task(update_acquisition_state_label(self.__state_controller.acquisition_state_model.value))
 
         def display_name_changed(display_name: str) -> None:
             self.__display_name = display_name
@@ -1095,10 +1081,6 @@ class CameraDisplayPanelController:
             self.__abort_button_enabled = enabled
             update_abort_button()
 
-        def data_item_states_changed(data_item_states: typing.Sequence[typing.Mapping[str, typing.Any]]) -> None:
-            self.__data_item_states = list(data_item_states)
-            update_status_text()
-
         def update_capture_button(visible: bool, enabled: bool) -> None:
             if visible:
                 capture_button.enabled = enabled
@@ -1130,16 +1112,19 @@ class CameraDisplayPanelController:
         self.__state_controller.on_binning_values_changed = None
         self.__state_controller.on_play_button_state_changed = play_button_state_changed
         self.__state_controller.on_abort_button_state_changed = abort_button_state_changed
-        self.__state_controller.on_data_item_states_changed = data_item_states_changed
         self.__state_controller.on_capture_button_state_changed = update_capture_button
         self.__state_controller.on_display_new_data_item = display_new_data_item
 
+        self.__acquisition_state_changed_listener = self.__state_controller.acquisition_state_model.property_changed_event.listen(acquisition_state_changed)
+
         play_button_canvas_item.on_button_clicked = self.__state_controller.handle_play_pause_clicked
         abort_button_canvas_item.on_button_clicked = self.__state_controller.handle_abort_clicked
         capture_button.on_button_clicked = self.__state_controller.handle_capture_clicked
 
         self.__state_controller.initialize_state()
 
+        acquisition_state_changed("value")
+
         checkstate = self.__show_processed_checkbox.check_state if self.__show_processed_checkbox else "unchecked"
 
         show_processed_checkbox_changed(checkstate)
@@ -1149,6 +1134,7 @@ class CameraDisplayPanelController:
         self.__display_panel = typing.cast(typing.Any, None)
         self.__state_controller.close()
         self.__state_controller = typing.cast(typing.Any, None)
+        self.__acquisition_state_changed_listener = typing.cast(typing.Any, None)
 
     def save(self, d: typing.MutableMapping[str, typing.Any]) -> None:
         d["hardware_source_id"] = self.__hardware_source_id
